/**
 * Copyright (C) 2007-2009 Felipe Contreras
 *
 * Purple is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 */

#include "pecan_contact.h"
#include "pecan_contact_priv.h"
#include "pecan_contactlist_priv.h"
#include "pecan_log.h"
#include "pecan_util.h"

#include "cvr/slp.h"
#include "pecan_ud.h" /* for pecan_ud_manager_contact_set_object */

#include <string.h>
#include <stdbool.h>

#ifdef HAVE_LIBPURPLE
#include "fix_purple.h"

/* libpurple stuff. */
#include "fix_purple_win32.h"
#include <cipher.h>
#include <account.h>
#endif /* HAVE_LIBPURPLE */

PecanContact *
pecan_contact_new (PecanContactList *contactlist)
{
    PecanContact *contact;

    contact = g_new0 (PecanContact, 1);

    contact->contactlist = contactlist;
    contact->groups = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);

    return contact;
}

void
pecan_contact_free (PecanContact *contact)
{
    if (contact->clientcaps)
        g_hash_table_destroy (contact->clientcaps);

    g_hash_table_destroy (contact->groups);

#if defined(PECAN_CVR)
    if (contact->msnobj)
        msn_object_destroy (contact->msnobj);
#endif /* defined(PECAN_CVR) */

    g_free (contact->passport);
    g_free (contact->friendly_name);
    g_free (contact->personal_message);
    g_free (contact->store_name);
    g_free (contact->guid);
    g_free (contact->phone.home);
    g_free (contact->phone.work);
    g_free (contact->phone.mobile);

    g_free (contact);
}

void
pecan_contact_update (PecanContact *contact)
{
#ifdef HAVE_LIBPURPLE
    PurpleAccount *account;
    const char *pidgin_status;
    gboolean idle = FALSE;

    g_return_if_fail (contact->contactlist);

    account = msn_session_get_user_data (contact->contactlist->session);

    switch (contact->status)
    {
        case PECAN_STATUS_OFFLINE:
            pidgin_status = "offline"; break;
        case PECAN_STATUS_BUSY:
            pidgin_status = "busy"; break;
        case PECAN_STATUS_BRB:
            pidgin_status = "brb"; break;
        case PECAN_STATUS_AWAY:
            pidgin_status = "away"; break;
        case PECAN_STATUS_PHONE:
            pidgin_status = "phone"; break;
        case PECAN_STATUS_LUNCH:
            pidgin_status = "lunch"; break;
        case PECAN_STATUS_HIDDEN:
            pidgin_status = "invisible"; break;
        case PECAN_STATUS_IDLE:
            idle = TRUE;
        case PECAN_STATUS_ONLINE:
            pidgin_status = "available"; break;
        default:
            pidgin_status = NULL; break;
    }

    purple_prpl_got_user_status (account, contact->passport, pidgin_status, NULL);

    if (contact->mobile && contact->status == PECAN_STATUS_OFFLINE)
        purple_prpl_got_user_status (account, contact->passport, "mobile", NULL);
    else
        purple_prpl_got_user_status_deactive (account, contact->passport, "mobile");

    purple_prpl_got_user_idle (account, contact->passport, idle, idle ? -1 : 0);
#endif /* HAVE_LIBPURPLE */
}

gboolean
pecan_contact_is_account (PecanContact *contact)
{
    if (strcmp (msn_session_get_username (contact->contactlist->session), contact->passport) == 0)
        return TRUE;

    return FALSE;
}

void
pecan_contact_set_state (PecanContact *contact,
                         const gchar *state)
{
    PecanStatus status;

    if (!state)
        status = PECAN_STATUS_OFFLINE;
    else if (strcmp (state, "NLN") == 0)
        status = PECAN_STATUS_ONLINE;
    else if (strcmp (state, "BSY") == 0)
        status = PECAN_STATUS_BUSY;
    else if (strcmp (state, "IDL") == 0)
        status = PECAN_STATUS_IDLE;
    else if (strcmp (state, "BRB") == 0)
        status = PECAN_STATUS_BRB;
    else if (strcmp (state, "AWY") == 0)
        status = PECAN_STATUS_AWAY;
    else if (strcmp (state, "PHN") == 0)
        status = PECAN_STATUS_PHONE;
    else if (strcmp (state, "LUN") == 0)
        status = PECAN_STATUS_LUNCH;
    else if (strcmp (state, "HDN") == 0)
        status = PECAN_STATUS_HIDDEN;
    else
        status = PECAN_STATUS_WRONG;

    contact->status = status;
}

void
pecan_contact_set_passport (PecanContact *contact,
                            const gchar *passport)
{
    g_return_if_fail (contact);

    g_free (contact->passport);
    contact->passport = pecan_normalize (passport);

    if (contact->contactlist)
    {
        g_hash_table_insert (contact->contactlist->contact_names,
                             g_strdup (passport), contact);
    }
}

void
pecan_contact_set_client_id (PecanContact *contact,
                             gulong client_id)
{
    contact->client_id = client_id;
}

gulong
pecan_contact_get_client_id (PecanContact *contact)
{
    return contact->client_id;
}

void
pecan_contact_set_friendly_name (PecanContact *contact,
                                 const gchar *name)
{
    g_return_if_fail (contact);

    pecan_debug ("passport=[%s],name=[%s]", contact->passport, name);

    if (contact->friendly_name && name &&
        strcmp (contact->friendly_name, name) == 0)
    {
        return;
    }

#ifdef HAVE_LIBPURPLE
    PurpleAccount *account;
    account = msn_session_get_user_data (contact->contactlist->session);

    g_return_if_fail (contact->contactlist);

    if (purple_account_get_bool (account, "hide_msgplus_tags", TRUE))
    {
        char* parsed_name;

        parsed_name = remove_plus_tags_from_str (name);

        if (!parsed_name)
            parsed_name = g_strdup (name);
        if (contact->friendly_name && parsed_name &&
            strcmp (contact->friendly_name, parsed_name) == 0)
            return;

        g_free (contact->friendly_name);
        contact->friendly_name = parsed_name;
    }
    else
    {
        g_free (contact->friendly_name);
        contact->friendly_name = g_strdup (name);
    }

    purple_buddy_set_public_alias (purple_account_get_connection (account),
                                   contact->passport, contact->friendly_name);

    /** @todo temporarily disable this until we have proper server-side aliases
     * support. */
#if 0
    /* If contact == account; display and friendly are the same thing. */
    /** @todo this is a libpurple specific thing */
    if (pecan_contact_is_account (contact))
    {
        pecan_debug ("contact is account");
        pecan_contact_set_store_name (contact, name);
    }
#endif
#else
    g_free (contact->friendly_name);
    contact->friendly_name = g_strdup (name);
#endif /* HAVE_LIBPURPLE */
}

void
pecan_contact_set_personal_message (PecanContact *contact,
                                    const gchar *value)
{
    g_return_if_fail (contact);

    pecan_debug ("passport=[%s],value=[%s]", contact->passport, value);

    if (contact->personal_message && value &&
        strcmp (contact->personal_message, value) == 0)
    {
        return;
    }

#ifdef HAVE_LIBPURPLE
    PurpleAccount *account;
    account = msn_session_get_user_data (contact->contactlist->session);

    g_return_if_fail (contact->contactlist);

    if (value && purple_account_get_bool (account, "hide_msgplus_tags", TRUE))
    {
        char* parsed_value;

        parsed_value = remove_plus_tags_from_str (value);

        if (!parsed_value)
            parsed_value = g_strdup (value);
        if (contact->personal_message && parsed_value &&
            strcmp (contact->personal_message, parsed_value) == 0)
            return;

        g_free (contact->personal_message);
        contact->personal_message = parsed_value;
    }
    else
    {
        g_free (contact->personal_message);
        contact->personal_message = g_strdup (value);
    }
#else
    g_free (contact->personal_message);
    contact->personal_message = g_strdup (value);
#endif /* HAVE_LIBPURPLE */
}

void
pecan_contact_set_current_media (PecanContact *contact,
                                 const gchar *current_media)
{
    gchar **current_media_array;
    int strings = 0;

    g_return_if_fail (contact);
    g_return_if_fail (current_media);

    /*
    * 0: Application
    * 1: 'Music'/'Games'/'Office'
    * 2: Enabled
    * 3: Format
    * 4: Title
    * If 'Music':
    *  5: Artist
    *  6: Album
    *  7: ?
    */

    current_media_array = g_strsplit (current_media, "\\0", 0);

#if GLIB_CHECK_VERSION(2,6,0)
    strings  = g_strv_length (current_media_array);
#else
    while (current_media_array[++strings] != NULL);
#endif

    if (strings < 4)
    {
        current_media_array = g_strsplit (current_media, "&#x5C;0", 0);

#if GLIB_CHECK_VERSION(2,6,0)
        strings  = g_strv_length (current_media_array);
#else
        while (current_media_array[++strings] != NULL);
#endif
    }

    if (strings >= 4 && strcmp (current_media_array[2], "1") == 0)
    {
        if (strcmp (current_media_array[1], "Music") == 0)
            contact->media.type = CURRENT_MEDIA_MUSIC;
        else if (strcmp (current_media_array[1], "Games") == 0)
            contact->media.type = CURRENT_MEDIA_GAMES;
        else if (strcmp (current_media_array[1], "Office") == 0)
            contact->media.type = CURRENT_MEDIA_OFFICE;
        else
            contact->media.type = CURRENT_MEDIA_UNKNOWN;

        g_free (contact->media.title);
        if (strings == 4)
            contact->media.title = purple_unescape_html (current_media_array[3]);
        else
            contact->media.title = purple_unescape_html (current_media_array[4]);

        g_free (contact->media.artist);
        if (strings > 5)
            contact->media.artist = purple_unescape_html (current_media_array[5]);
        else
            contact->media.artist = NULL;

        g_free (contact->media.album);
        if (strings > 6)
            contact->media.album = purple_unescape_html (current_media_array[6]);
        else
            contact->media.album = NULL;
    }
    else
    {
        contact->media.type = CURRENT_MEDIA_UNKNOWN;

        g_free (contact->media.title);
        contact->media.title = NULL;

        g_free (contact->media.artist);
        contact->media.artist = NULL;

        g_free (contact->media.album);
        contact->media.album = NULL;
    }

    g_strfreev (current_media_array);
}

void
pecan_contact_set_store_name (PecanContact *contact,
                              const gchar *name)
{
    g_return_if_fail (contact);

    pecan_debug ("passport=[%s],name=[%s]", contact->passport, name);

    if (contact->contactlist)
    {
        if (msn_session_get_bool (contact->contactlist->session, "use_server_alias"))
        {
            /** @todo this is a hack to disable display names. */
            if (name &&
                strcmp (contact->passport, name) == 0)
            {
                name = NULL;
            }
        }
    }

    if (contact->store_name && name &&
        strcmp (contact->store_name, name) == 0)
    {
        return;
    }

    g_free (contact->store_name);
    contact->store_name = g_strdup (name);

#ifdef HAVE_LIBPURPLE
    g_return_if_fail (contact->contactlist);

    {
        PurpleAccount *account;
        PurpleConnection *connection;
        MsnSession *session;

        session = contact->contactlist->session;
        account = msn_session_get_user_data (session);
        connection = purple_account_get_connection (account);

        purple_buddy_set_private_alias (connection, contact->passport, contact->store_name);
    }

    /** @todo temporarily disable this until we have proper server-side aliases
     * support. */
#if 0
    /* If contact == account; display and friendly are the same thing. */
    /** @todo this is a libpurple specific thing */
    if (pecan_contact_is_account (contact))
    {
        pecan_debug ("contact is account");
        pecan_contact_set_friendly_name (contact, name);
    }
#endif
#endif /* HAVE_LIBPURPLE */
}

void
pecan_contact_set_guid (PecanContact *contact,
                        const gchar *guid)
{
    g_return_if_fail (contact);

    g_free (contact->guid);
    contact->guid = g_strdup (guid);
    if (contact->contactlist && guid)
    {
        g_hash_table_insert (contact->contactlist->contact_guids, g_strdup (guid), contact);
    }
}

void
pecan_contact_set_buddy_icon (PecanContact *contact,
                              PecanBuffer *image)
{
#if defined(PECAN_CVR)
#ifdef HAVE_LIBPURPLE
    MsnObject *msnobj;

    g_return_if_fail (contact);

    msnobj = msn_object_new_from_image (image, "TFR2C2.tmp", pecan_contact_get_passport (contact),
                                        MSN_OBJECT_USERTILE);
    pecan_contact_set_object (contact, msnobj);
#endif /* HAVE_LIBPURPLE */
#endif /* defined(PECAN_CVR) */
}

void
pecan_contact_add_group_id (PecanContact *contact,
                            const gchar *group_guid)
{
    const gchar *passport;

    g_return_if_fail (contact);

    passport = pecan_contact_get_passport (contact);

    pecan_debug ("passport=[%s],group_guid=[%s]", passport, group_guid);

    if (group_guid)
    {
        g_hash_table_insert (contact->groups, g_strdup (group_guid), "foo");
    }

#ifdef HAVE_LIBPURPLE
    g_return_if_fail (contact->contactlist);

    {
        PecanContactList *contactlist;
        PurpleAccount *account;
        PurpleBuddy *b = NULL;
        PurpleGroup *g = NULL;
        const gchar *group_name;

        contactlist = contact->contactlist;
        group_name = pecan_contactlist_find_group_name (contactlist, group_guid);
        account = msn_session_get_user_data (contactlist->session);

        /* If this contact is in the no-group, remove him, since now he is in a
         * group. */
        if (group_guid)
        {
            const gchar *t_group_name;
            PurpleGroup *t_g;

            t_group_name = pecan_contactlist_find_group_name (contactlist, NULL);
            t_g = purple_find_group (t_group_name);

            if (t_g)
            {
                b = purple_find_buddy_in_group (account, passport, t_g);

                if (b)
                {
                    purple_blist_remove_buddy (b);
                }
            }
        }

        if (group_name)
            g = purple_find_group (group_name);

        /* If the group is not there, add it */
        if (!g)
        {
            g = purple_group_new (group_name);
            purple_blist_add_group (g, NULL);
        }

        b = purple_find_buddy_in_group (account, passport, g);

        if (!b)
        {
            b = purple_buddy_new (account, passport, NULL);
            purple_blist_add_buddy (b, NULL, g, NULL);
        }

        b->proto_data = contact;
    }
#endif /* HAVE_LIBPURPLE */
}

void
pecan_contact_remove_group_id (PecanContact *contact,
                               const gchar *group_guid)
{
    g_return_if_fail (contact);
    g_return_if_fail (group_guid);

    pecan_debug ("passport=[%s],group_guid=[%s]", contact->passport, group_guid);

    g_hash_table_remove (contact->groups, group_guid);
}

guint
pecan_contact_get_group_count (PecanContact *contact)
{
    return g_hash_table_size (contact->groups);
}

void
pecan_contact_set_home_phone (PecanContact *contact,
                              const gchar *number)
{
    g_return_if_fail (contact);

    g_free (contact->phone.home);

    contact->phone.home = (!number ? NULL : g_strdup (number));
}

void
pecan_contact_set_work_phone (PecanContact *contact,
                              const gchar *number)
{
    g_return_if_fail (contact);

    g_free (contact->phone.work);

    contact->phone.work = (!number ? NULL : g_strdup (number));
}

void
pecan_contact_set_mobile_phone (PecanContact *contact,
                                const gchar *number)
{
    g_return_if_fail (contact);

    g_free (contact->phone.mobile);

    contact->phone.mobile = (!number ? NULL : g_strdup (number));
}

#if defined(PECAN_CVR)
void
pecan_contact_set_object (PecanContact *contact,
                          MsnObject *obj)
{
    g_return_if_fail (contact);

    /** @todo sometimes we need to force an update, in those cases the old and
     * new object will be the same, we must not free the object, so in order to
     * do that properly we need to implement msnobj ref/unref. */
    /* For now just assume there was no obj set. */
    if (contact->msnobj == obj)
        contact->msnobj = NULL;

    if (contact->msnobj)
        msn_object_destroy (contact->msnobj);

    contact->msnobj = obj;

    /** @todo make this a hook. */
    pecan_ud_manager_contact_set_object (contact, obj);
}
#endif /* defined(PECAN_CVR) */

void
pecan_contact_set_client_caps (PecanContact *contact,
                               GHashTable *info)
{
    g_return_if_fail (contact);
    g_return_if_fail (info);

    if (contact->clientcaps)
        g_hash_table_destroy (contact->clientcaps);

    contact->clientcaps = info;
}

const gchar *
pecan_contact_get_passport (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->passport;
}

const gchar *
pecan_contact_get_friendly_name (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->friendly_name;
}

const gchar *
pecan_contact_get_personal_message (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->personal_message;
}

const gchar *
pecan_contact_get_store_name (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->store_name;
}

const gchar *
pecan_contact_get_guid (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->guid;
}

const gchar *
pecan_contact_get_home_phone (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->phone.home;
}

const gchar *
pecan_contact_get_work_phone (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->phone.work;
}

const gchar *
pecan_contact_get_mobile_phone (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->phone.mobile;
}

#if defined(PECAN_CVR)
MsnObject *
pecan_contact_get_object (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->msnobj;
}
#endif /* defined(PECAN_CVR) */

GHashTable *
pecan_contact_get_client_caps (const PecanContact *contact)
{
    g_return_val_if_fail (contact, NULL);

    return contact->clientcaps;
}

static inline gboolean
is_blocked (PecanContact *contact)
{
    return ((contact->list_op & (1 << MSN_LIST_BL)) ? true : false);
}

static inline gboolean
is_offline (PecanContact *contact)
{
    return (contact->status == PECAN_STATUS_OFFLINE ? true : false);
}

gboolean
pecan_contact_can_receive (PecanContact *contact)
{
    if (is_blocked (contact))
	return false;

    if (is_offline (contact))
	return false;

    return true;
}
