/**
 * Copyright (C) 2007-2009 Felipe Contreras
 *
 * Purple is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 */

#include "pecan_status.h"
#include "pecan_log.h"

#include "session.h"
#include "notification.h"

#include "ab/pecan_contact.h"

#include "session_private.h"

#include "msn.h"

#include <string.h>

#ifdef HAVE_LIBPURPLE
/* libpurple stuff. */
#include "fix_purple_win32.h"
#include <account.h>
#endif /* HAVE_LIBPURPLE */

static inline const gchar *
util_type_to_str (PecanStatus status)
{
    static const gchar *status_text[] =
    { NULL, "NLN", "BSY", "IDL", "BRB", "AWY", "PHN", "LUN", "HDN", NULL };

    return status_text[status];
}

static inline void
pecan_set_personal_message (MsnSession *session,
                            gchar *value,
                            gchar *current_media)
{
    MsnCmdProc *cmdproc;
    gchar *payload;

    cmdproc = session->notification->cmdproc;
    payload = pecan_strdup_printf ("<Data><PSM>%s</PSM><CurrentMedia>%s</CurrentMedia></Data>", value ? value : "", current_media ? current_media : "");

    {
        MsnTransaction *trans;
        trans = msn_transaction_new (cmdproc, "UUX", "%d", strlen (payload));
        msn_transaction_set_payload (trans, payload, strlen (payload));
        msn_cmdproc_send_trans (cmdproc, trans);
    }
}

#ifdef HAVE_LIBPURPLE
static inline PecanStatus
util_status_from_session (MsnSession *session)
{
    PurpleAccount *account;
    PecanStatus msnstatus;
    PurplePresence *presence;
    PurpleStatus *status;
    const gchar *status_id;

    account = msn_session_get_user_data (session);
    presence = purple_account_get_presence (account);
    status = purple_presence_get_active_status (presence);
    status_id = purple_status_get_id (status);

    if (strcmp (status_id, "available") == 0)
        msnstatus = PECAN_STATUS_ONLINE;
    else if (strcmp (status_id, "away") == 0)
        msnstatus = PECAN_STATUS_AWAY;
    else if (strcmp (status_id, "brb") == 0)
        msnstatus = PECAN_STATUS_BRB;
    else if (strcmp (status_id, "busy") == 0)
        msnstatus = PECAN_STATUS_BUSY;
    else if (strcmp (status_id, "phone") == 0)
        msnstatus = PECAN_STATUS_PHONE;
    else if (strcmp (status_id, "lunch") == 0)
        msnstatus = PECAN_STATUS_LUNCH;
    else if (strcmp (status_id, "invisible") == 0)
        msnstatus = PECAN_STATUS_HIDDEN;
    else if (strcmp (status_id, "online") == 0)
    {
        if (purple_presence_is_idle (presence))
            msnstatus = PECAN_STATUS_IDLE;
        else
            msnstatus = PECAN_STATUS_ONLINE;
    }
    else
    {
        pecan_error ("wrong: status_id=[%s]", status_id);
        msnstatus = PECAN_STATUS_WRONG;
    }

    return msnstatus;
}

static gchar *
create_current_media_string (PurplePresence *presence)
{
    const gchar *title, *game, *office;

    PurpleStatus *status = purple_presence_get_status (presence, "tune");

    if (!status || !purple_status_is_active (status))
        return NULL;

    title = purple_status_get_attr_string (status, PURPLE_TUNE_TITLE);
    game = purple_status_get_attr_string (status, "game");
    office = purple_status_get_attr_string (status, "office");

    if (title) 
    {
        const gchar *artist = purple_status_get_attr_string (status, PURPLE_TUNE_ARTIST);
        const gchar *album = purple_status_get_attr_string (status, PURPLE_TUNE_ALBUM);

        return g_strdup_printf("WMP\\0Music\\01\\0{0}%s%s\\0%s\\0%s\\0%s\\0",
                               artist ? " - {1}" : "",
                               album ? " ({2})" : "",
                               title,
                               artist ? artist : "",
                               album ? album : "");
    }
    else if (game)
        return g_strdup_printf ("\\0Games\\01\\0Playing {0}\\0%s\\0", game);
    else if (office)
        return g_strdup_printf ("\\0Office\\01\\0Editing {0}\\0%s\\0", office);
    else
        return NULL;
}

void
pecan_update_status (MsnSession *session)
{
    MsnCmdProc *cmdproc;
    PecanContact *user;
    const gchar *state_text;

    g_return_if_fail (session);

    if (!session->logged_in)
        return;

    user = msn_session_get_contact (session);
    cmdproc = session->notification->cmdproc;
    state_text = util_type_to_str (util_status_from_session (session));

#if defined(PECAN_CVR)
    {
        MsnObject *msnobj;

        msnobj = pecan_contact_get_object (user);

        if (msnobj)
        {
            gchar *msnobj_str;

            msnobj_str = msn_object_to_string (msnobj);

            msn_cmdproc_send (cmdproc, "CHG", "%s %d %s", state_text,
                              MSN_CLIENT_ID, purple_url_encode (msnobj_str));

            g_free (msnobj_str);
        }
        else
        {
            msn_cmdproc_send (cmdproc, "CHG", "%s %d", state_text,
                              MSN_CLIENT_ID);
        }
    }
#else
    msn_cmdproc_send (cmdproc, "CHG", "%s %d", state_text,
                      MSN_CLIENT_ID);
#endif /* defined(PECAN_CVR) */
}

void
pecan_update_personal_message (MsnSession *session)
{
    PurpleAccount *account;
    PurplePresence *presence;
    gchar *current_media;

    g_return_if_fail (session);

    if (!session->logged_in)
        return;

    account = msn_session_get_user_data (session);
    presence = purple_account_get_presence (account);

    current_media = create_current_media_string (presence);

#ifndef PECAN_USE_PSM
    const gchar *msg;

    msg = purple_account_get_string (account, "personal_message", "");
    pecan_set_personal_message (session, (gchar *) msg, current_media);
#else
    PurpleStatus *status;
    const gchar *formatted_msg;

    status = purple_account_get_active_status (account);
    formatted_msg = purple_status_get_attr_string (status, "message");

    if (formatted_msg)
    {
        gchar *msg;
        gchar *tmp;

        tmp = purple_markup_strip_html (formatted_msg);
        msg = g_markup_escape_text (tmp, -1);
        pecan_set_personal_message (session, msg, current_media);

        g_free (tmp);
        g_free (msg);
    }
    else
    {
        pecan_set_personal_message (session, NULL, current_media);
    }
#endif /* PECAN_USE_PSM */

    if (current_media)
        g_free (current_media);
}
#endif /* HAVE_LIBPURPLE */

gboolean
pecan_timeout_tune_status (gpointer data)
{
    MsnSession *session;
    PurpleAccount *account;
    PurplePresence *presence;
    PurpleStatus *status;

    session = data;

    if (!session)
        return FALSE;

    account = msn_session_get_user_data (session);
    presence = purple_account_get_presence (account);
    status = purple_presence_get_status (presence, "tune");

    if (status)
    {
        if (session->autoupdate_tune.enabled)
        {
            pecan_update_personal_message (session);

            if (!status || !purple_status_is_active (status))
                session->autoupdate_tune.enabled = FALSE;
        }
        else
        {
            if (status && purple_status_is_active (status))
            {
                session->autoupdate_tune.enabled = TRUE;
                pecan_update_personal_message (session);
            }
        }
    }

    session->autoupdate_tune.timer = g_timeout_add_seconds (10, pecan_timeout_tune_status, session);

    return FALSE;
}
